'''
@package: dc
@author scorp
@link: http://hierarchical-cluster-engine.com/
@copyright: Copyright &copy; 2013-2014 IOIX Ukraine
@license: http://hierarchical-cluster-engine.com/license/
@since: 0.1
'''

import Constants
import logging
import dc.EventObjects

logger = logging.getLogger(Constants.LOGGER_NAME)


##FieldRecalculator class makes come common processing of databse fields recalculation (using in Task classes)
class FieldRecalculator(object):

  def __init__(self):
    pass


  ##commonSiteRecalculate - common recalculate method
  #
  #@param queryCallback function for queries execution
  #@param additionCause additional SQL cause
  #@param fieldName - processing field name (of `sites` tables)
  #@param siteId - site id
  def commonSiteRecalculate(self, additionCause, fieldName, siteId, queryCallback):
    tableName = Constants.DC_URLS_TABLE_NAME_TEMPLATE % siteId
    query = "SELECT COUNT(*) FROM %s WHERE %s" % (tableName, additionCause)
    res = queryCallback(query, Constants.SECONDARY_DB_ID)
    if res is not None and len(res) > 0 and len(res[0]) > 0:
      logger.debug(">>>RECALCULATOR Site.%s update to %s value", fieldName, str(res[0][0]))
      query = "UPDATE `sites` SET `%s`=%s WHERE `id` = '%s'" % (fieldName, str(res[0][0]), siteId)
      queryCallback(query, Constants.PRIMARY_DB_ID)


  ##siteResourcesRecalculate - recalculate sites.Resources field
  #
  def siteResourcesRecalculate(self, siteId, queryCallback):
    #self.commonSiteRecalculate(queryCallback, "State>3 AND Crawled>0", "Resources", siteId)
    #self.commonSiteRecalculate("Crawled>0 AND Size>0", "Resources", siteId, queryCallback)
    self.commonSiteRecalculate("`Status`= " + str(dc.EventObjects.URL.STATUS_CRAWLED) + \
                               " AND `Crawled`>0 AND `Size`>0 " + \
                               "AND ((`ErrorMask` & 4198399) = 0) AND `ContentType`='text/html'",
                               "Resources", siteId, queryCallback)

  ##siteContentsRecalculate - recalculate sites.Contents field
  #
  def siteContentsRecalculate(self, siteId, queryCallback):
    #self.commonSiteRecalculate(queryCallback, "State=7 AND Crawled>0 AND Processed>0", "Contents", siteId)
    self.commonSiteRecalculate("TagsCount>0", "Contents", siteId, queryCallback)


  # updateCollectedURLs updates sites.CollectedURLs field
  #
  # @param siteId - siteId
  # @param queryCallback - callback sql function
  def updateCollectedURLs(self, siteId, queryCallback):
    tableName = ("urls_%s" % str(siteId))
    QUERY_TEMPLATE = "UPDATE `sites` SET `CollectedURLs`=(SELECT count(*) FROM dc_urls.%s) WHERE `Id`='%s'"
    query = QUERY_TEMPLATE % (tableName, siteId)
    queryCallback(query, Constants.PRIMARY_DB_ID)


  # updateNewURLs updates sites.newURLs field
  #
  # @param siteId - siteId
  # @param queryCallback - callback sql function
  def updateNewURLs(self, siteId, queryCallback):
    tableName = ("urls_%s" % str(siteId))
    QUERY_TEMPLATE = "UPDATE `sites` SET `NewURLs`=(SELECT count(*) FROM dc_urls.%s WHERE `Status`=1) WHERE `Id`='%s'"
    query = QUERY_TEMPLATE % (tableName, siteId)
    queryCallback(query, Constants.PRIMARY_DB_ID)


  # updateDeletedURLs updates sites.deletedURLs field
  #
  # @param siteId - siteId
  # @param queryCallback - callback sql function
  def updateDeletedURLs(self, siteId, queryCallback):
    tableName = ("urls_%s" % str(siteId))
    QUERY_TEMPLATE_SELECT = "SELECT count(*) FROM %s"
    query = QUERY_TEMPLATE_SELECT % tableName
    res = queryCallback(query, Constants.FOURTH_DB_ID, Constants.EXEC_INDEX, True)
    if res is not None and len(res) > 0 and len(res[0]) > 0:
      count = int(res[0][0])
      QUERY_TEMPLATE_UPDATE = "UPDATE `sites` SET `DeletedURLs`=%s WHERE `Id`='%s'"
      query = QUERY_TEMPLATE_UPDATE % (str(count), siteId)
      queryCallback(query, Constants.PRIMARY_DB_ID)


  # commonRecalc method makes all recalculations
  #
  # @param siteId - siteId
  # @param queryCallback - callback sql function
  # @param recalcType - full or partial recalculating
  def commonRecalc(self, siteId, queryCallback, recalcType=dc.EventObjects.FieldRecalculatorObj.FULL_RECALC):
    self.siteResourcesRecalculate(siteId, queryCallback)
    self.siteContentsRecalculate(siteId, queryCallback)
    if recalcType == dc.EventObjects.FieldRecalculatorObj.FULL_RECALC:
      self.updateCollectedURLs(siteId, queryCallback)
    self.updateNewURLs(siteId, queryCallback)
    self.updateDeletedURLs(siteId, queryCallback)
