"""
HCE project, Python bindings, Distributed Tasks Manager application.
Event objects definitions.

@package: dc
@file ProcessorTask.py
@author Oleksii <developers.hce@gmail.com>
@link: http://hierarchical-cluster-engine.com/
@copyright: Copyright &copy; 2013-2014 IOIX Ukraine
@license: http://hierarchical-cluster-engine.com/license/
@since: 0.1
"""


import os
import sys
import glob
import time
import icu
import json
import hashlib
import re
try:
  import cPickle as pickle
except ImportError:
  import pickle
# import logging
import logging.config
import ConfigParser
import MySQLdb as mdb
import MySQLdb.cursors
# import sqlite3 as lite
from contextlib import closing
from cement.core import foundation
from subprocess import Popen
from subprocess import PIPE
from collections import namedtuple

from app.Utils import varDump
from app.Utils import PathMaker
import app.Utils
from dc_processor.ScraperInData import ScraperInData
from dc.EventObjects import URL
from dc.EventObjects import Site
from dc.EventObjects import Batch
import dc.EventObjects as dc_event
from dc import EventObjects
from dc_db.TasksManager import TasksManager as DBTasksManager
from dc_processor.ProcessorException import ProcessorException
import app.Utils as Utils  # pylint: disable=F0401
from dc_crawler.Constants import ERROR_MIME_NOT_WRITE
from dc_crawler.Constants import DB_URLS
from dc_processor.ScraperResponse import ScraperResponse
import app.Consts as APP_CONSTS
import dc.Constants as DC_CONSTS
import dc_processor.Constants as CONSTS
from dc_crawler.DBTasksWrapper import DBTasksWrapper
from app.Utils import SQLExpression
from scraper_utils import decode


APP_NAME = "processor-task"

DC_URLS_DB_NAME = "dc_urls"
DC_URLS_TABLE_PREFIX = "urls_"
DC_SITES_DB_NAME = "dc_sites"
DC_SITES_TABLE_NAME = "sites"
DC_URLS_TABLE_NAME = "urls"
DC_SITES_PROPERTIES_TABLE_NAME = "sites_properties"

MSG_ERROR_PROCESS_BATCH_ITEM = "Error process batch item "
MSG_ERROR_PROCESS_BATCH = "Error process batch. "
MSG_ERROR_LOAD_CONFIG = "Error loading config file. Exciting."
MSG_ERROR_LOAD_LOG_CONFIG_FILE = "Error loading logging config file. Exiting."
MSG_ERROR_LOAD_URL_DATA = "Can't load url data: "
MSG_ERROR_LOAD_SITE_DATA = "Error load site data: "
MSG_ERROR_READ_SITE_FROM_DB = "Error read site data from db"
MSG_ERROR_PROCESS_TASK = "Can't process task "
MSG_ERROR_SERIALISE_RESULT = "Error serialize result "
MSG_ERROR_GET_SITE_FILE_DB = "Error get site file db "
MSG_ERROR_UPDATE_RECORD = "Error update record "
MSG_ERROR_UPDATE_PROCESSED_URL = "Error update processed url "
MSG_ERROR_GET_RAW_CONTENT_FROM_DB = "Error get raw content from disk "
MSG_ERROR_PROCESS = "Error process "
MSG_ERROR_LOAD_SITE_PROPERTIES = "Error load site properties "
MSG_ERROR_CHECK_SITE = "Error check site. "
MSG_ERROR_LOAD_OPTIONS = "Error load options. "
MSG_ERROR_CONVERT_RAW_CONTENT_CHARSET = "Cannot convert raw content charset. "
MSG_ERROR_UPDATE_SITE_RESOURCES = "Error update site resources. "
MSG_ERROR_EMPTY_BATCH = "Error read input pickle from stdin."
MSG_ERROR_CHECK_CONTENT_HASH = "Fail to check content hash"
MSG_ERROR_CALC_CONTENT_HASH = "Fail to calc content hash"
MSG_ERROR_CHECK_CONTENT_HASH_DUPLICATE = "Can't check content hash duplicate"

MSG_INFO_PROCESSOR_CMD = "Processor cmd: "
MSG_INFO_LOAD_SITE_PROPERTIES = "Mismatch load site properties "
MSG_INFO_PROCESS_BATCH = "Skipped process batch. "
MSG_INFO_PROCESS_BATCH_ITEM = "Skipped process batch item "
MSG_INFO_PROCESSOR_EXIT_CODE = "Scraper exit_code: "
MSG_INFO_PROCESSOR_OUTPUT = "Scraper output: "
MSG_INFO_PROCESSOR_ERROR = "Scraper err: "

EXIT_SUCCESS = 0
EXIT_FAILURE = 1

ERROR_MASK_NO_ERRORS = 0
ERROR_MASK_SITE_OK = 0
ERROR_MASK_SITE_MAX_ERRORS = 1 << 32
ERROR_MASK_SITE_MAX_RESOURCES = 1 << 33
ERROR_MASK_SITE_UNSUPPORTED_CONTENT_TYPE = 1 << 34
ERROR_MASK_URL_ENCODING_ERROR = 1 << 35
ERROR_MASK_SCRAPER_ERROR = 1 << 36
ERROR_MASK_MISSED_RAW_CONTENT_ON_DISK = 1 << 37


TEMPLATE_SELECTION_TYPE_DEFAULT = 0
TEMPLATE_SELECTION_TYPE_FULL_URLMD5 = 1
TEMPLATE_SELECTION_TYPE_DOMAIN_URLMD5 = 2

URLS_OF_MEDIA_CONTENT = 1
ENV_PROCESSOR_STORE_PATH = "ENV_PROCESSOR_STORE_PATH"

Results = namedtuple("Results", "exit_code, output, err")
# scraper = None

# #The CrawlerTask class, is a interface for fetching content from the web
#
# This object is a run at once application
class ProcessorTask(foundation.CementApp):

  """
  # Configuration settings options names
  DB_HOST = "db_host"
  DB_PORT = "db_port"
  DB_USER = "db_user"
  DB_PWD = "db_pwd"
  DB_SITES = "db_dc_sites"
  DB_URLS = "db_dc_urls"
  """

  # Mandatory
  class Meta(object):
    label = APP_NAME
    def __init__(self):
      self.scraper_response = None
      self.input_batch = None
      self.pubdate = None
      self.timezone = None
      self.refine_tags = None
      self.contentHash = None
      self.list_hashed_tags = None
      self.contentHash = None
      self.contentURLMd5 = ""
      self.contentURLMd5Algorithm = None
      self.contentURLMd5Action = None
      self.db_task_ini = None
      self.wrapper = None

  # #constructor
  # initialize default fields
  def __init__(self):
    # call base class __init__ method
    foundation.CementApp.__init__(self)
    self.exit_code = EXIT_SUCCESS
    self.logger = None
    self.result_object = None
    self.site_properties = None
    self.template = None
    self.raw_content = None
    self.ErrorMask = None
    self.site = None
    self.DBConnector = None
    self.table_name = None
    self.url = None
    self.process_time = None
    self.batch_item = None
    self.dc_sites_db_connect = None
    self.dc_urls_db_connect = None
    self.site_connector = None
    self.URLConnector = None
    self.site_table = None
    self.URLTable = None
    self.raw_data_dir = None
    self.processCTypes = []
    self.charset = None
    self.filters = None
    self.processorName = None
    self.processorProperties = None
    self.wrapper = None
    self.batch_sites = set()


  # #setup
  # setup application
  def setup(self):
    # call base class setup method
    foundation.CementApp.setup(self)
    self.args.add_argument('-c', '--config', action='store', metavar='config_file', help='config ini-file')


  # #run
  # run application
  def run(self):
    # call base class run method
    foundation.CementApp.run(self)

    # config section
    self.loadConfig()

    # load scraper
    # self.loadScraper()

    # load logger config file
    self.loadLogConfigFile()

    # load mandatory options
    self.loadOptions()

    # load db backend
    self.loadDBBackend()

    # load sqlite db backend
    # self.loadSqliteDBBackend()

  """
  # #helper function for correct query execution
  #
  # @param query query for execution
  def executeQuery(self, query):
    self.logger.info("Connector: " + str(self.DBConnector))
    try:
      with closing(self.DBConnector.cursor(MySQLdb.cursors.DictCursor)) as cursor:
        cursor.execute(query)
        self.DBConnector.commit()
        return cursor.fetchall()
    except mdb.Error as err:  # @todo logging in db_task
      self.DBConnector.rollback()
      self.logger.error("%s %s" % (err.args[0], err.args[1]))
      raise err
  """

  # #loadSiteProperties
  #
  # @param site - site object
  def loadSiteProperties(self):
  # support of TEMPLATE_SELECTION_TYPE
    self.processorName = None
    self.site_properties = None
    self.processCTypes = []
    self.template = ""
    self.logger.debug("self.site.properties: " + str(self.site.properties))
    try:
      # Update site properties from batch item properties
      keys = [property["name"] for property in self.site.properties]
      self.logger.info("keys: %s" % str(keys))
      for key in self.batch_item.properties.keys():
        if key in keys:
          for property in self.site.properties:
            if property["name"]==key:
              property["value"] = self.batch_item.properties[key]
              self.logger.info("%s exist in site properties." % key)
              self.logger.info("Old value: %s" % varDump(property))
              self.logger.info("New value: %s" % varDump(property))
        else:
          self.logger.info("%s don't exist in site properties. Add property" % key)
          self.site.properties.append({"name":key, "value":self.batch_item.properties[key], "URLMd5":self.batch_item.urlId})
      self.logger.debug("Updated site's properties: " + str(self.site.properties))

      for property in self.site.properties:

        #TEMPLATE_SELECTION_TYPE
        if "TEMPLATE_SELECTION_TYPE" == property["name"]:
          self.template_selection_type = int(property["value"])
          self.logger.debug("TEMPLATE_SELECTION_TYPE: " + str(self.template_selection_type))
          self.logger.debug("HASH: %s, URL: %s" % (str(hashlib.md5(app.Utils.UrlParser.generateDomainUrl(self.url.url)).hexdigest()),self.url.url))

        #PROCESS_CTYPES
        elif "PROCESS_CTYPES" == property["name"]:
          self.processCTypes = property["value"]
          self.logger.debug("PROCESS_CTYPES: " + str(self.processCTypes))

        #TIMEZONE
        elif "TIMEZONE" == property["name"]:
          self.timezone = property["value"]
          self.logger.debug("TIMEZONE: " + str(self.timezone))

        #REFINE_TAGS
        elif "REFINE_TAGS" == property["name"]:
          self.refine_tags = property["value"].split(',')
          self.logger.debug("REFINE_TAGS: " + str(self.refine_tags))

        #PROCESSOR_NAME
        elif "PROCESSOR_NAME" == property["name"]:
          self.processorName = property["value"]
          self.logger.debug("PROCESSOR_NAME: " + str(self.processorName))

        #PROCESSOR_PROPERTIES
        elif "PROCESSOR_PROPERTIES" == property["name"]:
          self.processorProperties = property["value"]
          self.logger.debug("PROCESSOR_PROPERTIES: " + str(self.processorProperties))
      
        #CONTENT_HASH
        elif "CONTENT_HASH" == property["name"]:
          self.contentHash = property["value"]
          self.logger.debug("CONTENT_HASH: " + str(self.contentHash))

        #template
        elif "template" == property["name"]:
          if self.template_selection_type == TEMPLATE_SELECTION_TYPE_DEFAULT:
            self.template = property["value"]
          elif (self.template_selection_type == TEMPLATE_SELECTION_TYPE_FULL_URLMD5) and \
            property["URLMd5"] == self.batch_item.url.md5:
            self.template = property["value"]
          elif (self.template_selection_type == TEMPLATE_SELECTION_TYPE_DOMAIN_URLMD5) and \
            property["URLMd5"] == hashlib.md5(app.Utils.UrlParser.generateDomainUrl(self.url.url)).hexdigest():
            self.logger.debug("HASH: %s, URl: %s" % (str(hashlib.md5(app.Utils.UrlParser.generateDomainUrl(self.url.url)).hexdigest()),self.url.url))
            self.template = property["value"]
          self.logger.info("Template: " + varDump(self.template))
      
      #debug info
      self.logger.debug("+++++++++++++++")
      self.logger.debug("TEMPLATE_SELECTION_TYPE: " + str(self.template_selection_type))
      self.logger.debug("HASH: %s, URL: %s" % (str(hashlib.md5(app.Utils.UrlParser.generateDomainUrl(self.url.url)).hexdigest()),self.url.url))
      self.logger.debug("PROCESS_CTYPES: " + str(self.processCTypes))
      self.logger.debug("TIMEZONE: " + str(self.timezone))
      self.logger.debug("REFINE_TAGS: " + str(self.refine_tags))
      self.logger.debug("PROCESSOR_NAME: " + str(self.processorName))
      self.logger.debug("CONTENT_HASH: " + str(self.contentHash))
      self.logger.info("Template: " + varDump(self.template))
      self.logger.info("Template type: " + str(type(self.template)))
      self.logger.debug("+++++++++++++++")

      # ckeck contentType and PROCESS_CTYPES
      if self.url.contentType not in self.processCTypes:
        self.ErrorMask |= ERROR_MASK_SITE_UNSUPPORTED_CONTENT_TYPE
        self.updateURLRecordFailed()
        raise ProcessorException("url ContentType not matched! url.contentType: <<%s>>, \
                                 site_properties.PROCESS_CTYPES: <<%s>>"\
                                 % (self.url.contentType, str(self.processCTypes)))
    except ProcessorException, err:
      self.logger.info(MSG_INFO_LOAD_SITE_PROPERTIES + str(err.message))
      raise err
    except Exception as err:
      self.logger.error(MSG_ERROR_LOAD_SITE_PROPERTIES + str(err.message))
      raise err
    
    """
    if "TEMPLATE_SELECTION_TYPE" in self.SITE.properties:
      self.template_selection_type = int(self.SITE.properties["TEMPLATE_SELECTION_TYPE"])
      self.logger.debug("TEMPLATE_SELECTION_TYPE: " + str(self.template_selection_type))
      self.logger.debug("HASH: %s, URL: %s" % (str(hashlib.md5(app.Utils.UrlParser.generateDomainUrl(self.url.url)).hexdigest()),self.url.url))
    # Added by Oleksii
    # support PROCESSOR_NAME functionality
    # set self.processorName to None to prevent using value from previous batch item
    self.DBConnector = self.dc_sites_db_connect
    # query = "SELECT Value FROM `%s` \
    #        WHERE `Name`='template' \
    #        AND `Site_Id`='%s'" \
    #        % (DC_SITES_PROPERTIES_TABLE_NAME, \
    #        self.batch_item.siteId)
    query = "SELECT * FROM `%s` \
            WHERE `Site_Id`='%s'" \
            % (DC_SITES_PROPERTIES_TABLE_NAME, \
            self.batch_item.siteId)
    self.logger.info("query: " + query)
    try:
      #rows = self.executeQuery(query)
      #self.logger.info("rows: " + str(rows))
      #for row in rows:
        self.logger.info("row: " + str(row))
        if row.has_key("Name") and row["Name"] == "PROCESS_CTYPES":
          # self.processCTypes.append(row["Value"])
          self.processCTypes = row["Value"]
          self.logger.info("self.processCTypes: " + str(self.processCTypes))
          self.logger.info("url contentType: " + str(self.url.contentType))

        # support of timezone
        if row.has_key("Name") and row["Name"] == "TIMEZONE":
          self.timezone = row["Value"]
          self.logger.debug("site has timezone: " + str(self.timezone))

        # support of refine tags
        if row.has_key("Name") and row["Name"] == "REFINE_TAGS":
          self.refine_tags = row["Value"].split(',')
          self.logger.debug("site has refine_tags: %s", str(self.refine_tags))

        # support of PROCESSOR_NAME
        if row.has_key("Name") and row["Name"] == "PROCESSOR_NAME":
          self.processorName = row["Value"]
          self.logger.debug("PROCESSOR_NAME: " + str(self.processorName))

        # support of CONTENT_HASH
        if row.has_key("Name") and row["Name"] == "CONTENT_HASH":
            self.contentHash = row["Value"]
            self.logger.debug("CONTENT_HASH: " + str(self.contentHash))

        if row.has_key("Name") and row["Name"] == "template":
          if self.template_selection_type == TEMPLATE_SELECTION_TYPE_DEFAULT:
            self.template = row["Value"]
          elif (self.template_selection_type == TEMPLATE_SELECTION_TYPE_FULL_URLMD5) and \
            row["URLMd5"] == self.batch_item.url.md5:
            self.template = row["Value"]
          elif (self.template_selection_type == TEMPLATE_SELECTION_TYPE_DOMAIN_URLMD5) and \
            row["URLMd5"] == hashlib.md5(app.Utils.UrlParser.generateDomainUrl(self.url.url)).hexdigest():
            self.logger.debug("HASH: %s, URl: %s" % (str(hashlib.md5(app.Utils.UrlParser.generateDomainUrl(self.url.url)).hexdigest()),self.url.url))
            self.template = row["Value"]
          self.logger.info("Template: " + str(self.template))
      # ckeck contentType and PROCESS_CTYPES
      if self.url.contentType not in self.processCTypes:
        self.ErrorMask |= ERROR_MASK_SITE_UNSUPPORTED_CONTENT_TYPE
        self.updateURLRecordFailed()
        raise ProcessorException("url ContentType not matched! url.contentType: <<%s>>, \
                                site_properties.PROCESS_CTYPES: <<%s>>"\
                                % (self.url.contentType, str(self.processCTypes)))
    except ProcessorException, err:
      self.logger.info(MSG_INFO_LOAD_SITE_PROPERTIES + err.message)
      raise err
    except Exception as err:
      self.logger.error(MSG_ERROR_LOAD_SITE_PROPERTIES + err.message)
      raise err
    """


  # #
  #
  def getProcessorCmd(self):
    # support PROCESSOR_NAME
    cmd = CONSTS.PYTHON_BINARY + " " + CONSTS.SCRAPER_BINARY + " " + CONSTS.SCRAPER_CFG
    if self.processorName is None or self.processorName == CONSTS.PROCESSOR_EMPTY:
      cmd = CONSTS.PYTHON_BINARY + " " + CONSTS.SCRAPER_BINARY + " " + CONSTS.SCRAPER_CFG
    elif self.processorName == CONSTS.PROCESSOR_STORE:
      cmd = CONSTS.PYTHON_BINARY + " " + CONSTS.STORE_PROCESSOR_BINARY + " " + CONSTS.STORE_PROCESSOR_CFG
    elif self.processorName == CONSTS.PROCESSOR_FEED_PARSER:
      cmd = CONSTS.PYTHON_BINARY + " " + CONSTS.PROCESSOR_FEED_PARSER_BINARY + " " + CONSTS.PROCESSOR_FEED_PARSER_CFG
    self.logger.debug(MSG_INFO_PROCESSOR_CMD + cmd)
    return cmd


  # #
  #
  def refineResults(self):
    flag = True
    try:
      self.logger.debug("Refine has started")
      for refiner in self.refine_tags:
        self.doRefine(refiner)
    except Exception as err:
      self.logger.debug("Refine has failed: %s", str(err))
    return flag


  # #
  #
  def doRefine(self, refiner):
    flag = True
    try:
      self.logger.debug("Refiner is: %s", str(refiner))
      cmd = ""
      process = Popen(cmd, stdout=PIPE, stdin=PIPE, stderr=PIPE, shell=True, close_fds=True)
      (output, err) = process.communicate(input=self.result_object)
    except Exception as err:
      self.logger.debug("Refine has failed: %s", str(err))
    return flag


  # #
  #
  def process(self):
    try:
      scraper_input_object = ScraperInData(self.url.url, \
                                           self.batch_item.urlId, \
                                           self.batch_item.siteId, \
                                           self.raw_content, \
                                           self.template, \
                                           self.filters, \
                                           self.url.lastModified, \
                                           self.timezone, \
                                           self.batch_item,
                                           self.processorProperties)
      # self.logger.info("scraper_input_object: " + str(vars(scraper_input_object)))
      input_pickled_object = pickle.dumps(scraper_input_object)
      Utils.storePickleOnDisk(input_pickled_object, ENV_PROCESSOR_STORE_PATH, "processor.out." + self.batch_item.urlId)
      # self.logger.info("input_pickled_object: " + str(input_pickled_object))

      # sleep to reduce system load
      time.sleep(self.url.processingDelay / 1000.0)

      # support PROCESSOR_NAME
      cmd = self.getProcessorCmd()
      process = Popen(cmd, stdout=PIPE, stdin=PIPE, stderr=PIPE, shell=True, close_fds=True)
      # scraper_response = Popen(cmd, stdin=PIPE, stdout=sys.stdout, shell=True).communicate(input=input_pickled_object)
      (output, err) = process.communicate(input=input_pickled_object)
      # self.logger.debug("scraper_response pickle: %s", str(output))
      self.logger.debug("scraper_response error: %s", str(err))
      # open("scraper_input.bin", "w").write(input_pickled_object)
      exit_code = process.wait()
      self.logger.debug("scraper_response exit_code: %s", str(exit_code))
      # finish processing if scraper return exit_code EXIT_FAILURE
      if exit_code == EXIT_FAILURE:
        self.logger.error("Scraper has failed.")
        raise ProcessorException("Scraper has failed.")

      self.scraper_response = pickle.loads(output)
      self.logger.debug("scraper_response: %s", str(self.scraper_response))
      self.logger.debug("tagsCount: %s", str(self.scraper_response.tagsCount))
      self.logger.debug("tagsMask: %s", str(self.scraper_response.tagsMask))
      #self.logger.debug("processedContent: %s", self.scraper_response.processedContent)
      self.logger.info(MSG_INFO_PROCESSOR_EXIT_CODE + str(exit_code))
      # self.logger.info(MSG_INFO_PROCESSOR_OUTPUT + str(output))
      self.logger.info(MSG_INFO_PROCESSOR_ERROR + str(err))


      self.result_object = output
    except Exception as err:
      tbi = Utils.getTracebackInfo()
      self.logger.error(MSG_ERROR_PROCESS + str(vars(err)) + "\n" + tbi)
      raise err
    return Results(exit_code, output, err)


  # #
  #
  def getRawContentFromFS(self):
    self.raw_content = None
    try:
      urlContentObj = dc_event.URLContentRequest(self.batch_item.siteId, self.batch_item.urlObj.url, contentTypeMask=dc_event.URLContentRequest.CONTENT_TYPE_RAW_LAST + dc_event.URLContentRequest.CONTENT_TYPE_RAW)
      self.raw_content = self.wrapper.urlContent([urlContentObj])
      #self.logger.debug("self.raw_content: " + varDump(self.raw_content[0].rawContents[0].buffer))
      self.raw_content = decode(self.raw_content[0].rawContents[0].buffer)
      """
      if len(self.batch_item.siteId):
        path = self.raw_data_dir + "/" + \
                self.batch_item.siteId + "/" + \
                PathMaker(self.batch_item.urlId).getDir() + "/*.bin"
        # self.logger.info("path: "+str(path))
        # file_name = sorted(glob.glob(path))[-1]
        # self.logger.info("file_name: "+str(file_name))
      else:
        path = self.raw_data_dir + "/0/" + PathMaker(self.batch_item.urlId).getDir() + "*.bin"
      self.logger.info("path: " + str(path))
      # file_name = sorted(glob.glob(path))[-1]
      file_name = max(glob.glob(path), key=os.path.getctime)
      self.logger.info("file_name: " + str(file_name))
      self.raw_content = open(file_name, "rb").read()
      """
    except Exception as err:
      tbi = Utils.getTracebackInfo()
      self.logger.error(str(vars(err)) + "\n" + tbi)
      self.ErrorMask = ERROR_MASK_MISSED_RAW_CONTENT_ON_DISK
      self.logger.error(MSG_ERROR_GET_RAW_CONTENT_FROM_DB + err.message)
      raise ProcessorException(MSG_ERROR_GET_RAW_CONTENT_FROM_DB)


  # #
  #
  def convertRawContentCharset(self):
    try:
      self.charset = icu.CharsetDetector(self.raw_content).detect().getName()  # pylint: disable=E1101
      self.logger.info("Charset incoming raw data: " + self.charset)
      self.raw_content = self.raw_content.decode(self.charset)
    except Exception, err:
      self.logger.debug(MSG_ERROR_CONVERT_RAW_CONTENT_CHARSET + err.message)
      self.ErrorMask |= ERROR_MASK_URL_ENCODING_ERROR
      self.updateURLRecordFailed()
      raise ProcessorException(MSG_ERROR_CONVERT_RAW_CONTENT_CHARSET)


  # #
  #
  def updateURLCharset(self):
    try:
      urlUpdateObj = dc_event.URLUpdate(self.batch_item.siteId, self.batch_item.urlId, dc_event.URLStatus.URL_TYPE_MD5)
      urlUpdateObj.UDate = SQLExpression("NOW()")
      urlUpdateObj.charset = self.charset
      updated_rows = self.wrapper.urlUpdate(urlUpdateObj)
      """
      self.DBConnector = self.URLConnector
      query = "UPDATE `%s` \
        SET `Charset`='%s', \
        `UDate`=NOW() \
        WHERE `URLMd5`='%s'" \
        % (self.URLTable, \
        self.charset, \
        self.batch_item.urlId)
      self.logger.info("query: " + query)
      self.executeQuery(query)
      """
    except Exception as err:
      self.logger.error(MSG_ERROR_UPDATE_PROCESSED_URL + err.message)
      raise err


  # readFilters read pattern from sites_filters
  #
  def readFilters(self):
    #self.DBConnector = self.site_connector
    #siteId = self.batch_item.siteId if self.batch_item.siteId else "0"
    #query = "SELECT `Type`, `Pattern` FROM `sites_filters` WHERE `Site_Id` = '%s' AND `Mode`=%s" \
    #% (siteId, URLS_OF_MEDIA_CONTENT)
    #self.logger.debug("query: " + query)
    #self.filters = self.executeQuery(query)
    self.filters = self.site.filters
    self.logger.debug("sites_filters: " + varDump(self.filters))


  # #
  #
  def updateProcessedURL(self):
    # check url state
    state = URL.STATE_ENABLED
    processed_time = int((time.time() - self.process_time) * 1000)

    if self.scraper_response:
      tagsMask = self.scraper_response.tagsMask or 0
      tagsCount = self.scraper_response.tagsCount or 0
      self.pubdate = self.scraper_response.pubdate or None
      #self.contentMd5 = self.scraper_response.contentMd5 or None
    else:
      tagsMask = 0
      tagsCount = 0
    # updated by Oleksii
    # we shouldn't set state to error
    # cause it checked and rejected after many algorithms
    # if self.ErrorMask != ERROR_MASK_SITE_OK:
    #  state = URL.STATE_ERROR

    try:
      urlUpdateList = []
      urlUpdateObj = dc_event.URLUpdate(self.batch_item.siteId, self.batch_item.urlId, dc_event.URLStatus.URL_TYPE_MD5)
      urlUpdateObj.status = URL.STATUS_PROCESSED
      urlUpdateObj.state = state
      urlUpdateObj.errorMask = SQLExpression("`ErrorMask` + %s" % str(self.ErrorMask))
      urlUpdateObj.tagsMask = tagsMask
      urlUpdateObj.tagsCount = tagsCount
      urlUpdateObj.totalTime = SQLExpression(("`TotalTime` + %s" % str(processed_time)))
      urlUpdateObj.processingTime = processed_time
      urlUpdateObj.pDate = str(self.pubdate) if self.pubdate is not None else None
      urlUpdateObj.UDate = SQLExpression("NOW()")
      urlUpdateObj.tcDate = SQLExpression("NOW()")
      urlUpdateObj.contentURLMd5 = self.contentURLMd5
      urlUpdateList.append(urlUpdateObj)
      #updated_rows = self.wrapper.urlUpdate(urlUpdateObj)

      if (self.processorName == CONSTS.PROCESSOR_FEED_PARSER) \
        and (dc_event.BatchItem.PROP_FEED in self.batch_item.properties):
        for url, value in self.batch_item.properties[dc_event.BatchItem.PROP_FEED].items():
          urlUpdateObj = dc_event.URLUpdate(self.batch_item.siteId, value["urlMd5"], dc_event.URLStatus.URL_TYPE_MD5)
          urlUpdateObj.status = URL.STATUS_PROCESSED
          urlUpdateObj.state = state
          urlUpdateObj.errorMask = self.ErrorMask
          urlUpdateObj.tagsMask = tagsMask
          urlUpdateObj.tagsCount = tagsCount
          urlUpdateObj.totalTime = SQLExpression(("`TotalTime` + %s" % str(processed_time)))
          urlUpdateObj.processingTime = processed_time
          urlUpdateObj.pDate = str(self.pubdate)
          urlUpdateObj.UDate = SQLExpression("NOW()")
          urlUpdateObj.contentURLMd5 = self.contentURLMd5
          urlUpdateList.append(urlUpdateObj)
      result = self.wrapper.urlUpdate(urlUpdateList)
      """
      self.DBConnector = self.URLConnector
      if self.pubdate is not None:
        query = "UPDATE `%s` SET `Status`=%s, `State`=%s, `ErrorMask`=%s, `TagsMask`=%s, `TagsCount`=%s, \
          `TotalTime`=`TotalTime`+%s, `ProcessingTime`=%s, `PDate`='%s', `UDate`=NOW(), `ContentURLMD5`='%s' WHERE `URLMd5`='%s'" \
          % (self.URLTable, URL.STATUS_PROCESSED, state, self.ErrorMask, tagsMask, tagsCount, processed_time, \
          processed_time, self.pubdate, self.contentURLMd5, self.batch_item.urlId)
      else:
        query = "UPDATE `%s` SET `Status`=%s, `State`=%s, `ErrorMask`=%s, `TagsMask`=%s, `TagsCount`=%s, \
          `TotalTime`=`TotalTime`+%s, `ProcessingTime`=%s, `UDate`=NOW(), `ContentURLMD5`='%s' WHERE `URLMd5`='%s'" \
          % (self.URLTable, URL.STATUS_PROCESSED, state, self.ErrorMask, tagsMask, tagsCount, processed_time, \
          processed_time, self.contentURLMd5, self.batch_item.urlId)
      self.logger.info("query: " + query)
      self.executeQuery(query)
      """
    except Exception as err:
      self.logger.error(MSG_ERROR_UPDATE_PROCESSED_URL + err.message)
      raise err


  # addResource add Resources counter for sites table
  def updateSiteResources(self):
    try:
      localSiteUpdate = dc_event.SiteUpdate(self.batch_item.siteId)
      for attr in localSiteUpdate.__dict__:
        if hasattr(localSiteUpdate, attr):
          setattr(localSiteUpdate, attr, None)
      localSiteUpdate.id = self.batch_item.siteId
      localSiteUpdate.tcDate = SQLExpression("NOW()")
      #localSiteUpdate.contents = SQLExpression("`Contents` + 1")
      updated_count = self.wrapper.siteNewOrUpdate(localSiteUpdate)

      """
      # increment resources if it is first processing, but not re-crawling
      if (self.url.processed == 0) and (self.ErrorMask == ERROR_MASK_SITE_OK):
        self.DBConnector = self.site_connector
        query = "UPDATE %s SET `Contents` = `Contents` + 1 WHERE `Id` = '%s'" % \
        (self.site_table, self.batch_item.siteId)
        self.logger.info("query: " + query)
        self.executeQuery(query)
      """
    except Exception as err:
      self.logger.error(MSG_ERROR_UPDATE_SITE_RESOURCES + err.message)
      raise err


  # # processContentHash
  #
  def processContentHash(self):
    try:
      if self.contentHash:
        self.logger.debug("Site has content hash rule: %s", str(self.contentHash))
        self.contentHash = json.loads(self.contentHash)
        self.list_hashed_tags = self.contentHash["tags"].split(",")
        self.contentURLMd5Algorithm = self.contentHash["algorithm"]
        self.contentURLMd5Action = self.contentHash["delete"]
        self.logger.debug("List hashed tags: %s", str(self.list_hashed_tags))
        self.calcContentHash()
        self.checkContentHashDuplicate()
        self.logger.debug("content hash buffer: %s", str(self.hashed_content))
      else:
        self.logger.debug("Site hasn't content hash rule: %s", str(self.contentHash))
    except ProcessorException as err:
      self.logger.debug(MSG_ERROR_CALC_CONTENT_HASH + " : " + err.message)


  ## checkContentHashDuplicate
  #
  def checkContentHashDuplicate(self):
    #self.DBConnector = self.URLConnector
    #self.table_name = self.URLTable
    try:
      query = "SELECT * FROM `%s` WHERE `ContentURLMd5`='%s'" % (self.URLTable, self.contentURLMd5)
      row = self.wrapper.customRequest(query, DB_URLS)
      """
      #query = "SELECT * FROM `%s` WHERE `ContentURLMd5`='%s' AND `URLMd5`<>'%s' AND `ContentURLMD5`<>''" \
      #  % (self.table_name, self.contentURLMd5, self.batch_item.urlId)
      query = "SELECT * FROM `%s` WHERE `ContentURLMd5`='%s'" \
        % (self.table_name, self.contentURLMd5)
      self.logger.info("query: " + query)
      row = self.executeQuery(query)
      self.logger.info("row: " + str(row))
      """
      if len(row):
        self.logger.info("Duplicate exist: %s", str(row))
        if self.contentURLMd5Action == CONSTS.CONTENT_HASH_ACTION_DELETE:
          self.deleteDuplicate()
      else:
        self.logger.info("Duplicate doesn't exist: %s", str(row))
    except ProcessorException as err:
      self.logger.debug(MSG_ERROR_CHECK_CONTENT_HASH_DUPLICATE + " : " + err.message)


  # # checkContentHashDuplicate
  #
  def deleteDuplicate(self):
    # Create new URLDelete object
    urlDelete = EventObjects.URLDelete(self.batch_item.siteId, self.batch_item.urlId, EventObjects.URLStatus.URL_TYPE_MD5)
    cfgParser = ConfigParser.ConfigParser()
    cfgParser.read(self.db_task_ini)
    dbTask = DBTasksManager(cfgParser)
    drceSyncTasksCoverObj = DC_CONSTS.DRCESyncTasksCover(DC_CONSTS.EVENT_TYPES.URL_DELETE, [urlDelete])
    responseDRCESyncTasksCover = dbTask.process(drceSyncTasksCoverObj)


  # # getContentHash
  #
  def getContentHash(self):
    if self.contentURLMd5Algorithm == CONSTS.CONTENT_HASH_ALGORITHM_MD5:
      self.contentURLMd5 = str(hashlib.md5(self.hashed_content).hexdigest())
    elif self.contentURLMd5Algorithm == CONSTS.CONTENT_HASH_ALGORITHM_MD5_WITHOUT_HTML:
      self.stripHTML()
      self.contentURLMd5 = str(hashlib.md5(self.hashed_content).hexdigest())
    else:
      self.logger.debug("Algorithm <<%s>> for calculating contentURLMd5 not supported.", str(self.contentURLMd5Algorithm))


  # # calcContentHash
  #
  def calcContentHash(self):
    self.hashed_content = ""
    self.contentURLMd5 = ""
    try:
      if self.contentURLMd5Algorithm != CONSTS.CONTENT_HASH_ALGORITHM_EMPTY:
        tag_list = json.loads(self.scraper_response.processedContent)["data"]["tagList"]
        #self.logger.debug("tag_list: <<%s>>", str(tag_list))
        for tag in tag_list:
          #self.logger.debug("tag: %s", str(tag))
          if tag["name"] in self.list_hashed_tags:
            self.logger.debug("Tag name added to hash: %s", str(tag["name"]))
            self.logger.debug("Tag value added to hash: %s", str(tag["data"]))
            self.hashed_content = self.hashed_content + str(tag["data"])
          else:
            self.logger.debug("Tag: %s not added to hash", tag["name"])
        self.getContentHash()
    except ProcessorException as err:
      self.logger.debug(MSG_ERROR_CHECK_CONTENT_HASH + " : " + err.message)


  # # calcContentHash
  #
  def stripHTML(self):
    # remove non-printable characters
    self.hashed_content = re.sub("(\n*)(\t*)(\r*)", "", self.hashed_content)
    # remove whole javascript
    self.hashed_content = re.sub("<script(.*?)>(.*?)</script>", "", self.hashed_content)
    # remove comments
    self.hashed_content = re.sub("<!--(.*?)-->", "", self.hashed_content)
    #remove HTML tags
    self.hashed_content = re.sub("<(.*?)>", "", self.hashed_content)
    #remove spaces
    self.hashed_content = re.sub(" ", "", self.hashed_content)


  # #processTask
  #
  def processTask(self):
    try:
      self.updateURLCharset()
      self.readFilters()
      self.process()
      if self.refine_tags:
        self.refineResults()
    except ProcessorException as err:
      self.logger.debug(MSG_ERROR_PROCESS_TASK + " : " + err.message)
      self.ErrorMask = ERROR_MASK_SCRAPER_ERROR
      raise err
    except Exception as err:
      self.logger.error(MSG_ERROR_PROCESS_TASK + " : " + err.message)
      tbi = Utils.getTracebackInfo()
      self.logger.error(str(vars(err)) + "\n" + tbi)
      self.ErrorMask = ERROR_MASK_SCRAPER_ERROR


  # #updateURL
  #
  def updateURL(self):
    try:
      # updated by Oleksii
      # if insert into kv db wasn't successfully
      # not update Processed counter
      processed_counter = 1
      if self.ErrorMask == ERROR_MASK_SCRAPER_ERROR:
        processed_counter = 0
      urlUpdateObj = dc_event.URLUpdate(self.batch_item.siteId, self.batch_item.urlId, dc_event.URLStatus.URL_TYPE_MD5)
      urlUpdateObj.processed = SQLExpression("`Processed`+" + str(processed_counter))
      urlUpdateObj.tcDate = SQLExpression("NOW()")
      urlUpdateObj.status = URL.STATUS_PROCESSING
      self.wrapper.urlUpdate(urlUpdateObj)
      """
      self.DBConnector = self.URLConnector
      query = "UPDATE `%s` \
        SET `Status`=%s, \
        `Processed`=`Processed`+%s \
        WHERE `URLMd5`='%s'" \
        % (self.URLTable, \
        URL.STATUS_PROCESSING, \
        processed_counter, \
        self.batch_item.urlId)
      self.logger.info("query: " + query)
      self.executeQuery(query)
      """
    except Exception as err:
      self.logger.error(MSG_ERROR_LOAD_SITE_DATA + err.message)
      raise err


  # #update site params
  # update site params
  # @param site - site object
  def updateSite(self):
    try:
      localSiteUpdate = dc_event.SiteUpdate(self.batch_item.siteId)
      for attr in localSiteUpdate.__dict__:
        if hasattr(localSiteUpdate, attr):
          setattr(localSiteUpdate, attr, None)
      localSiteUpdate.id = self.batch_item.siteId
      localSiteUpdate.tcDate = SQLExpression("NOW()")
      localSiteUpdate.errorMask = SQLExpression("`ErrorMask` | " + str(self.ErrorMask))
      localSiteUpdate.state = Site.STATE_SUSPENDED
      updated_count = self.wrapper.siteNewOrUpdate(localSiteUpdate)
      """
      self.DBConnector = self.site_connector
      query = "UPDATE `%s` \
        SET `ErrorMask`= `ErrorMask` | %s, \
        `Errors`=`Errors`+1, \
        `TcDate`=Now(), \
        `State`=%s \
        WHERE `Id`='%s'" \
        % (self.site_table, \
           self.ErrorMask, \
        Site.STATE_SUSPENDED, \
        self.batch_item.siteId)
      self.logger.info("query: " + query)
      self.executeQuery(query)
      """
    except Exception as err:
      self.logger.error(MSG_ERROR_LOAD_SITE_DATA + err.message)
      raise err


  # #update site params
  # update site params
  # @param site - site object
  def updateURLRecordFailed(self):
    try:
      #self.DBConnector = self.URLConnector

      # updated by Oleksii
      # we shouldn't set state to error
      # cause it checked and rejected after many algorithms
      # query = "UPDATE `%s` SET `ErrorMask`= `ErrorMask` | %s, `UDate`=Now(), `State`=%s WHERE `URLMd5`='%s'" \
      #  % (self.URLTable, self.ErrorMask, URL.STATE_ERROR, self.batch_item.urlId)
      urlUpdateObj = dc_event.URLUpdate(self.batch_item.siteId, self.batch_item.urlId, dc_event.URLStatus.URL_TYPE_MD5)
      urlUpdateObj.errorMask = SQLExpression("`ErrorMask` | " + str(self.ErrorMask))
      urlUpdateObj.UDate = SQLExpression("NOW()")
      self.wrapper.urlUpdate(urlUpdateObj)
      """
      query = "UPDATE `%s` SET `ErrorMask`= `ErrorMask` | %s, `UDate`=Now() WHERE `URLMd5`='%s'" \
        % (self.URLTable, self.ErrorMask, self.batch_item.urlId)

      self.logger.info("query: " + query)
      self.executeQuery(query)
      """
    except Exception as err:
      self.logger.error(MSG_ERROR_LOAD_SITE_DATA + err.message)
      raise err


  # #check site params
  # check site params
  # @param site - site object
  def checkSite(self):
    #self.ErrorMask = ERROR_MASK_SITE_OK
    if self.site.state == Site.STATE_DISABLED:
      raise ProcessorException("Site state is not active! Actual site state is: %s. " % self.site.state)

    # pass checking if it is first processing, but not re-crawling
    if self.site.maxResources and self.site.contents >= self.site.maxResources and self.url.processed == 0:  # pylint: disable=C0326
      self.ErrorMask |= ERROR_MASK_SITE_MAX_RESOURCES
      self.logger.debug("Site maxResources is reached! Site contents is: %s. Site maxResources: %s " \
                        % (self.site.contents, self.site.maxResources))
      if self.ErrorMask != ERROR_MASK_SITE_OK:
        raise ProcessorException(MSG_ERROR_CHECK_SITE)


  # #
  #
  def readSiteFromDB(self):
    try:
      #Add support db-task Site
      siteStatus = dc_event.SiteStatus(self.batch_item.siteId)
      drceSyncTasksCoverObj = DC_CONSTS.DRCESyncTasksCover(DC_CONSTS.EVENT_TYPES.SITE_STATUS, siteStatus)
      responseDRCESyncTasksCover = self.wrapper.process(drceSyncTasksCoverObj)
      self.site = responseDRCESyncTasksCover.eventObject

      #self.DBConnector = self.site_connector
      #self.site_table = DC_SITES_TABLE_NAME
      #query = "SELECT * FROM `%s` \
      #  WHERE `Id`='%s'" \
      #  % (self.site_table, \
      #  self.batch_item.siteId)
      #self.logger.info("query: " + query)
      #row = self.executeQuery(query)
      #self.logger.info("row: " + str(row))
      #if len(row):
      #  self.site = Site(self.url.url)
      #  self.site.state = row[0]["State"]
      #  self.site.errors = row[0]["Errors"]
      #  self.site.maxErrors = row[0]["MaxErrors"]
      #  self.site.resources = row[0]["Resources"]
      #  self.site.contents = row[0]["Contents"]
      #  self.site.maxResources = row[0]["MaxResources"]
      #  self.site.uDate = row[0]["UDate"]
      #  self.site.tcDate = row[0]["TcDate"]
      #  self.site.iterations = row[0]["Iterations"]
      #  self.site.priority = row[0]["Priority"]
      #  self.site.maxURLs = row[0]["MaxURLs"]
      #  self.site.maxResourceSize = row[0]["MaxResourceSize"]
      #  self.site.requestDelay = row[0]["RequestDelay"]
      #  self.site.httpTimeout = row[0]["HTTPTimeout"]
      #  self.site.errorMask = row[0]["ErrorMask"]
      #  self.site.urlType = row[0]["URLType"]
      #else:  # throw if site doesn't exists
      #  raise Exception("Site entry doesn't exist in table " + self.site_table)
    except Exception, err:
      self.logger.error(MSG_ERROR_READ_SITE_FROM_DB + str(err.message))
      raise err
    #self.logger.info("site object: " + varDump(self.site))


  # #
  #
  def loadSite(self):
    try:
      if not len(self.batch_item.siteId):
        self.batch_item.siteId = "0"
      self.readSiteFromDB()
    except Exception as err:
      self.logger.error(MSG_ERROR_LOAD_SITE_DATA + err.message)
      raise err


  # #
  #
  #
  def readUrlFromDB(self):
    urlStatus = dc_event.URLStatus(self.batch_item.siteId, self.batch_item.urlId)
    urlStatus.urlType = dc_event.URLStatus.URL_TYPE_MD5
    drceSyncTasksCoverObj = DC_CONSTS.DRCESyncTasksCover(DC_CONSTS.EVENT_TYPES.URL_STATUS, [urlStatus])
    responseDRCESyncTasksCover = self.wrapper.process(drceSyncTasksCoverObj)
    row = responseDRCESyncTasksCover.eventObject
    #self._url = None
    #if len(row):
    #  self._url = row[0]

    #self.DBConnector = self.URLConnector
    self.table_name = self.URLTable
    # self.url = None
    try:
      #query = "SELECT * FROM `%s` \
      #  WHERE `URLMd5`='%s'" \
      #  % (self.table_name, \
      #  self.batch_item.urlId)
      #self.logger.info("query: " + query)
      #row = self.executeQuery(query)
      #self.logger.info("row: " + str(row))
      if len(row):
        self.url = row[0]
        if self.url.errorMask & ERROR_MIME_NOT_WRITE:
          raise ProcessorException("MIME Content-Type not allow store content to disk")
      else:  # throw if url doesn't exists
        raise ProcessorException("URL entry doesn't exist in table " + str(self.table_name))
    except Exception, err:
      self.logger.debug(MSG_ERROR_LOAD_URL_DATA)
      raise err
    self.logger.info("url object: " + varDump(self.url))


  # #
  #
  def loadURL(self):
    # do siteId
    self.URLConnector = self.dc_urls_db_connect
    if len(self.batch_item.siteId):
      self.URLTable = DC_URLS_TABLE_PREFIX + self.batch_item.siteId
    else:
      self.URLTable = DC_URLS_TABLE_PREFIX + "0"
    self.logger.info("db backend Table for siteId %s is: %s" % (self.batch_item.siteId, self.URLTable))
    self.readUrlFromDB()


  def resetVars(self):
    self.scraper_response = None
    self.input_batch = None
    self.pubdate = None
    self.timezone = None
    self.refine_tags = None
    self.contentHash = None
    self.list_hashed_tags = None
    self.contentHash = None
    self.contentURLMd5 = ""
    self.contentURLMd5Algorithm = None
    self.contentURLMd5Action = None
    self.ErrorMask = ERROR_MASK_SITE_OK
    self.template_selection_type = TEMPLATE_SELECTION_TYPE_DEFAULT
    self.processorProperties = None


  # #process task
  # main resource processing
  # item is the batch item
  # @param item - the batch item
  def processBatchItem(self, batch_item):
    if not batch_item: return batch_item
    start_batch_item_time = time.time()
    response = None
    self.process_time = time.time()
    self.batch_item = batch_item
    self.logger.info("Batch item: siteId: %s" % (batch_item.siteId))
    self.logger.info("Batch item: urlId: %s" % (batch_item.urlId))
    self.logger.info("Batch item: url: %s" % (batch_item.urlObj.url))
    try:
      # update site list from batch
      self.batch_sites.update([batch_item.urlObj.siteId])

      self.resetVars()
      self.loadURL()
      self.loadSite()
      self.checkSite()

      #Check if Real-Time crawling resource already crawled
      if self.url.processed!=0 and \
        self.url.errorMask==ERROR_MASK_NO_ERRORS and \
        self.url.tagsCount>0 and \
        self.ErrorMask==ERROR_MASK_NO_ERRORS:
        self.logger.debug("Real time crawling. Check reprocessing.")
        self.logger.debug("Batch item properties: %s" % varDump(batch_item.properties))
        if (CONSTS.REPROCESS_KEY in batch_item.properties) and \
          (batch_item.properties[CONSTS.REPROCESS_KEY]==CONSTS.REPROCESS_VALUE_NO):
          self.logger.debug("RealTime Crawling: Cashed resource. Resource crawled and error mask is emppty. Don't need to reprocess.")
          return batch_item
        else:
          self.logger.debug("RealTime Crawling: Cashed resource. Resource crawled and error mask is emppty but properties reprocess is Yes or empty. Send to reprocessing.")

      #Check if content stored on disk
      if self.batch_item.urlObj.contentMask!=dc_event.URL.CONTENT_STORED_ON_DISK:
        self.logger.debug("Content not found on disk. Exit.")
        return batch_item

      self.loadSiteProperties()
      self.updateURL()
      self.getRawContentFromFS()
      self.convertRawContentCharset()
      self.processTask()
      self.processContentHash()
      self.updateSiteResources()
      response = batch_item
      batch_item_processing_time = int((time.time() - start_batch_item_time) * 1000)
      self.logger.debug("Batch item processing time: %s msec." % (batch_item_processing_time))
    except ProcessorException, err:
      self.logger.info(MSG_INFO_PROCESS_BATCH_ITEM + err.message)
    except Exception as err:
      self.logger.error(MSG_ERROR_PROCESS_BATCH_ITEM + err.message)
      tbi = Utils.getTracebackInfo()
      self.logger.error(str(vars(err)) + "\n" + tbi)
      self.exit_code = EXIT_FAILURE
    self.updateProcessedURL()
    #self.logger.info("response: " + varDump(response))
    return response


  # #process batch
  # the main processing of the batch object
  def processBatch(self):
    try:
      start_batch_time = time.time()

      # read pickled batch object from stdin and unpickle it
      #self.logger.debug("Start to read from stdin")
      input_pickled_object = sys.stdin.read()
      #self.logger.debug("Input pickled object:\n %s", str(input_pickled_object))
      #self.logger.debug("End to read from stdin")

      input_batch = pickle.loads(input_pickled_object)
      self.input_batch = input_batch
      Utils.storePickleOnDisk(input_pickled_object, ENV_PROCESSOR_STORE_PATH, "processor.in." + str(self.input_batch.id))
      if self.input_batch.id == 0:
        raise Exception("Empty batch items: maybe previous app had failed. ")
      self.logger.info("input_batch items: " + str(self.input_batch.items))
      # TODO main processing over every url from list of urls in the batch object
      batch_items = [self.processBatchItem(batch_item) for batch_item in self.input_batch.items]
      self.logger.info("Output batch items: " + str(batch_items))
      if input_batch.crawlerType == dc_event.Batch.TYPE_REAL_TIME_CRAWLER:
        process_task_batch = input_batch
      else:
        process_task_batch = Batch(input_batch.id, batch_items)
      self.logger.info("process_task_batch: " + varDump(process_task_batch))
      # Recalculate Contents
      self.wrapper.ContentsRecalculating(self.batch_sites)

      # send response to the stdout
      batch_processing_time = int((time.time() - start_batch_time) * 1000)
      self.logger.debug("Batch processing time: %s msec." % (batch_processing_time))
    except ProcessorException, err:
      self.logger.info(MSG_INFO_PROCESS_BATCH + err.message)
    except Exception.EOFError, err:  # pylint: disable-all
      self.logger.error(MSG_ERROR_EMPTY_BATCH)
    except Exception as err:
      self.logger.error(MSG_ERROR_PROCESS_BATCH + err.message, exc_info=True)
    output_pickled_object = pickle.dumps(process_task_batch)
    Utils.storePickleOnDisk(output_pickled_object, ENV_PROCESSOR_STORE_PATH, "processor.out." + str(input_batch.id))
    sys.stdout.write(output_pickled_object)
    sys.stdout.flush()
    # print pickle.dumps(process_task_batch)


  # #load db backend
  # load from cli argument or default config file
  def loadDBBackend(self):
    try:
      className = self.__class__.__name__
      dbHost = self.config.get(className, CONSTS.DB_HOST)
      dbPort = int(self.config.get(className, CONSTS.DB_PORT))
      dbUser = self.config.get(className, CONSTS.DB_USER)
      dbPWD = self.config.get(className, CONSTS.DB_PWD)
      db_dc_sites = self.config.get(className, CONSTS.DB_SITES)
      db_dc_urls = self.config.get(className, CONSTS.DB_URLS)

      self.dc_sites_db_connect = mdb.connect(dbHost, dbUser, dbPWD, db_dc_sites, dbPort)
      self.dc_urls_db_connect = mdb.connect(dbHost, dbUser, dbPWD, db_dc_urls, dbPort)
      self.site_connector = self.dc_sites_db_connect
      self.URLConnector = self.dc_urls_db_connect
    except Exception, err:
      self.logger.error(CONSTS.MSG_ERROR_LOAD_DB_BACKEND)


  # #load config from file
  # load from cli argument or default config file
  def loadConfig(self):
    try:
      self.config = ConfigParser.ConfigParser()
      self.config.optionxform = str
      self.config.read(self.pargs.config)
    except Exception, err:
      print MSG_ERROR_LOAD_CONFIG + err.message
      raise err


  # #load logging
  # load logging configuration (log file, log level, filters)
  #
  def loadLogConfigFile(self):
    try:
      log_conf_file = self.config.get("Application", "log")
      logging.config.fileConfig(log_conf_file)
      # logging.basicConfig()
      # self.logger = logging.getLogger(APP_NAME)
      # Logger initialization
      self.logger = logging.getLogger(APP_CONSTS.LOGGER_NAME)

    except Exception, err:
      print MSG_ERROR_LOAD_LOG_CONFIG_FILE + err.message
      raise err


  # #load mandatory options
  # load mandatory options
  #
  def loadOptions(self):
    try:
      self.raw_data_dir = self.config.get(self.__class__.__name__, "raw_data_dir")
      self.db_task_ini = self.config.get(self.__class__.__name__, "db-task_ini")
      # Add support operations updateCollectedURLs and removeURLs
      cfgParser = ConfigParser.ConfigParser()
      cfgParser.read(self.db_task_ini)
      self.wrapper = DBTasksWrapper(cfgParser)
    except Exception, err:
      print MSG_ERROR_LOAD_OPTIONS + err.message
      raise err


  # #get exit code
  # return extt code
  # 0 if no critical errors was encountered during processing
  # 1 if smth bad happened
  #
  def getExitCode(self):
    return self.exit_code
