#!/bin/bash

DIALOG=${DIALOG=dialog}
BACKTITLE='Install Dependencies for HCE project components: hce-node, DTM and DC'

RED='\e[0;31m'
BLUE='\e[1;34m'
DEF='\e[0m'

MYSQL_USER=hce
MYSQL_PWD=hce12345

SCRIPT_NAME=$(basename $0)

TEMPFILE=`mktemp /tmp/$SCRIPT_NAME.XXXXX`
trap "rm -f $TEMPFILE" 0 1 2 5 15
TMPDIR=`mktemp -d /tmp/$SCRIPT_NAME.XXXXXX`
trap "rm -rf $TMPDIR" 0 1 2 5 15

# FUNCTIONS #

function infoDialog() {
    # $1 - title
    # $2 - info messages
    # $2 - height
    if [[ -z $3 ]]
    then
        HEIGHT=5
    else
        HEIGHT=$3
    fi
    $DIALOG --backtitle "$BACKTITLE" --title "$1" \
            --colors --msgbox "$2" $HEIGHT 80
}

function textDialog() {
    # $1 - title
    # $2 - info messages
    $DIALOG --backtitle "$BACKTITLE" --title "$1" --exit-label "NEXT"\
            --textbox "$2" 20 80
}

function indicatorDialog() {
    # $1 - info messages
    # $2 - info %%
    $DIALOG --backtitle "$BACKTITLE" --title "$1" \
            --gauge "$1" 20 80 "$2"

}

function checkSudo() {
    if [[ $(whoami) != root ]]
    then
        infoDialog "Error" "Run script with root permissions or sudo: sudo $0 \"USERNAME\""
        clear
        exit 1
    fi
}

function checkOs() {
    if [[ ! -f '/etc/debian_version' ]]
    then
        infoDialog "Error" "This script is only for the Debian system! Cancelled."
        clear
        exit 1
    fi
}

function checkHomeDir() {
    # $1 = hce user
    if [[ ! -d /home/$1 ]]
    then
        infoDialog "Error" "Home dir (/home/${HCE_USER}) for user ${HCE_USER} not exists! Cancelled."
        clear
        exit 1
    fi
}

function start() {
    $DIALOG --backtitle "$BACKTITLE" --title "$1" \
            --checklist "Check HCE-project dependencies to install" 20 80 9 0 "The en_US.UTF-8 locale (if not present)" on \
                                                1 "PHP language API and management tools" on \
                                                2 "DC service" on \
                                                3 "Python language API and management tools" on \
                                                4 "The pyzmq library with zmq version 4.0.3 from source" on \
                                                5 "The phpmyadmin, latest" on \
                                                6 "Copy test site files to the apache virtual host DocumentRoot" on \
                                                7 "The mysql DB support (server, client, databases, user and permissions)" on \
                                                8 "Key-Value storage support for default site" on 2> $TEMPFILE
    case $? in
        0)
            CHECK=$(cat $TEMPFILE);;
        1)
            infoDialog "Select components" "Cancel. Exit" && clear && exit 1;;
        255)
            infoDialog "Select components" "Cancel. Exit" && clear && exit 1;;
    esac
}

function locales() {
    if [[ -z $(checkLocale) ]]
    then
        echo -e '#\nLANG=en_US.UTF-8\n' >> /etc/default/locale
        sed -i 's/# en_US.UTF-8 UTF-8/en_US.UTF-8 UTF-8/' /etc/locale.gen
        (
            COUNT=99
            echo "XXX"
            echo $COUNT
            echo "generation of locales..."
            echo "XXX"
            locale-gen
        ) | indicatorDialog "Added locales" "Generation of locales..."
        infoDialog "Added locales" "Locale en_US.UTF-8 added" && clear
    else
        infoDialog "Added locales" "Locale en_US.UTF-8 present" && clear
    fi
}

function dtse_php() {
    (
        COUNT=30
        echo "XXX"
        echo $COUNT
        echo "runnint apt-get update..."
        echo "XXX"
        apt-get update -qq
        COUNT=60
        echo "XXX"
        echo $COUNT
        echo "runnint apt-get install..."
        echo "XXX"
        apt-get install -y --force-yes bsdutils unzip libzmq3-dev apache2 php5 php5-dev php5-sqlite php-pear pkg-config libpgm-dev sphinxsearch make > $TEMPFILE
        COUNT=80
        echo "XXX"
        echo $COUNT
        echo "runnint install zmq-beta..."
        echo "XXX"
        echo '' >> $TEMPFILE
        pecl install zmq-beta >> $TEMPFILE
        if [[ -z $(checkZmqSo) ]]
        then
            COUNT=90
            echo "XXX"
            echo $COUNT
            echo "runninG add zmq module to php and reload apache"
            echo "XXX"
            echo -e "; configuration for php ZMQ module \n; priority=20 \nextension=zmq.so" > /etc/php5/cli/conf.d/20-zmq.ini
            service apache2 reload
        fi
        echo '' >> $TEMPFILE
        echo 'Done' >> $TEMPFILE
    ) | indicatorDialog "Install Demo Test Suit Environment for PHP language..." ""
    textDialog "Install Demo Test Suit Environment for PHP language..." "$TEMPFILE" && clear
}

function dtse_dc() {
    (
        COUNT=50
        echo "XXX"
        echo $COUNT
        echo "runnint apt-get update..."
        echo "XXX"
        apt-get update -qq
        COUNT=100
        echo "XXX"
        echo $COUNT
        echo "runnint apt-get install..."
        echo "XXX"
        apt-get install -y bc openjdk-7-jdk ruby php5-gmp g++ > $TEMPFILE
        echo '' >> $TEMPFILE
        echo 'Done' >> $TEMPFILE
    ) | indicatorDialog "Install Demo Test Suit Environment for DC..." ""
    textDialog "Install Demo Test Suit Environment for DC..." "$TEMPFILE" && clear
}

function dtse_python() {
    (
        COUNT=30
        echo "XXX"
        echo $COUNT
        echo "runnint apt-get update..."
        echo "XXX"
        apt-get update -qq
        COUNT=70
        echo "XXX"
        echo $COUNT
        echo "runnint apt-get install..."
        echo "XXX"
        apt-get install -y libpgm-dev mysql-server-5.5 python-pip python-dev python-flask python-flaskext.wtf libffi-dev libxml2-dev libxslt1-dev mysql-client libmysqlclient-dev python-mysqldb libicu-dev libgmp3-dev libtidy-dev libjpeg-dev > $TEMPFILE
        COUNT=80
        echo "XXX"
        echo $COUNT
        echo "runnint pip install..."
        echo "XXX"
        echo '' >> $TEMPFILE
        pip install cement sqlalchemy Flask-SQLAlchemy scrapy gmpy pyzmq lepl requests mysql-python pyicu newspaper goose-extractor pytidylib uritools python-magic feedparser Ghost.py pillow >> $TEMPFILE
        COUNT=90
        echo "XXX"
        echo $COUNT
        echo "runnint pip install pyzmq (with option zmq=bundle)..."
        echo "XXX"
        echo '' >> $TEMPFILE
        pip install pyzmq --install-option="--zmq=bundled" >> $TEMPFILE
        COUNT=100
        echo "XXX"
        echo $COUNT
        echo "runnint pip install urlnorm..."
        echo "XXX"
        echo '' >> $TEMPFILE
        pip install -U urlnorm >> $TEMPFILE
        echo '' >> $TEMPFILE
        echo 'Done' >> $TEMPFILE
    ) | indicatorDialog "Install Demo Test Suit Environment for Python language..." ""
    textDialog "Install Demo Test Suit Environment for Python language..." "$TEMPFILE" && clear
}

function pyzmq() {
    (
        COUNT=20
        echo "XXX"
        echo $COUNT
        echo "download zeromq v 4.0.4..."
        echo "XXX"
        cd $TMPDIR
        wget -q http://download.zeromq.org/zeromq-4.0.4.tar.gz
        COUNT=40
        echo "XXX"
        echo $COUNT
        echo "untar zeromq v 4.0.4..."
        echo "XXX"
        tar -xf zeromq-4.0.4.tar.gz > $TEMPFILE
        cd zeromq-4.0.4 2>>$TEMPFILE
        COUNT=60
        echo "XXX"
        echo $COUNT
        echo "configure, make and make install zeromq..."
        echo "XXX"
        echo '' >> $TEMPFILE
        ./configure -q 2>>$TEMPFILE >> $TEMPFILE && make -s >> $TEMPFILE && make -s install >> $TEMPFILE
        COUNT=90
        echo "XXX"
        echo $COUNT
        echo "install pyzmq with zmq version 4.0.3..."
        echo "XXX"
        echo '' >> $TEMPFILE
        ldconfig >> $TEMPFILE
        echo '' >> $TEMPFILE
        pip install -q pyzmq --install-option="--zmq=4.0.3" >> $TEMPFILE
        COUNT=100
        echo "XXX"
        echo $COUNT
        echo "remove tmp files..."
        echo "XXX"
        cd $TMPDIR
        rm -rf $TMPDIR/*
        echo '' >> $TEMPFILE
        echo 'Done' >> $TEMPFILE
     ) | indicatorDialog "Install pyzmq with zmq v4.0.3 from source..." ""
    textDialog "Install pyzmq with zmq v4.0.3 from source..." "$TEMPFILE" && clear
}

function phpmyadmin() {
    (
        COUNT=30
        echo "XXX"
        echo $COUNT
        echo "download phpmyadmin latest..."
        echo "XXX"
        cd $TMPDIR
        wget -q https://sourceforge.net/projects/phpmyadmin/files/latest/download?source=files -O $TMPDIR/phpmyadmin-latest.zip
        if [[ -z $(file $TMPDIR/phpmyadmin-latest.zip | grep empty) ]]
        then
            COUNT=60
            echo "XXX"
            echo $COUNT
            echo "unzip phpmyadmin..."
            echo "XXX"
            unzip -q phpmyadmin-latest.zip > $TEMPFILE
            rm -f phpmyadmin-latest.zip >> $TEMPFILE
            COUNT=90
            echo "XXX"
            echo $COUNT
            echo "install phpmyadmin..."
            echo "XXX"
            echo '' >> $TEMPFILE
            PHPMYADMINDIR=$(ls)
            if [[ -d /var/www/pma ]]
            then
                rm -rf /var/www/pma
            fi
            mkdir /var/www/pma
            mv $TMPDIR/$PHPMYADMINDIR/* /var/www/pma/ 2>>$TEMPFILE
            mv $TMPDIR/$PHPMYADMINDIR/.[!.]* /var/www/pma/ 2>>$TEMPFILE
            cp /var/www/pma/config.sample.inc.php /var/www/pma/config.inc.php 2>>$TEMPFILE
            if [[ -z $(grep "LogiDEFookieValidity" /var/www/pma/config.inc.php) ]]
            then
                sed -i "s/?>/\$cfg['LogiDEFookieValidity'] = '86400'\;\n?>/" /var/www/pma/config.inc.php
            fi
            cd /tmp
        else
            echo "phpmyadmin not downloaded and not installed" > $TEMPFILE
        fi
        echo '' >> $TEMPFILE
        echo 'Done' >> $TEMPFILE
     ) | indicatorDialog "Install phpmyadmin..." ""
    textDialog "Install phpmyadmin..." "$TEMPFILE" && clear
}

function mvTestSite() {
    cp /home/${HCE_USER}/hce-node-bundle/api/python/data/ftests/test_site/* /var/www/ 2>$TEMPFILE
    chown -R www-data:www-data /var/www
    infoDialog "Set up test web site" "Test site copied and added permissions\n$(<$TEMPFILE)" 8 && clear
}

function mysqlSetup() {
    set -f
    if [[ $(pgrep mysql) ]]
    then
        MYPW=$(cat /etc/mysql/debian.cnf |grep password|uniq|awk '{ print ($3)}')
        CHECK_MYSQL_USER_LOCALHOST=$(echo "SHOW GRANTS FOR '$MYSQL_USER'@'localhost';"| mysql -udebian-sys-maint -p$MYPW 2>/dev/null && : || echo "NONE")
        CHECK_MYSQL_USER_ALL=$(echo "SHOW GRANTS FOR '$MYSQL_USER'@'%';"| mysql -udebian-sys-maint -p$MYPW 2>/dev/null && : || echo "NONE")

        if [[ $CHECK_MYSQL_USER_LOCALHOST == "NONE" ]]
        then
            echo "CREATE USER '$MYSQL_USER'@'localhost' IDENTIFIED BY '$MYSQL_PWD';"| mysql -udebian-sys-maint -p$MYPW
            echo "GRANT ALL ON *.* TO '$MYSQL_USER'@'localhost';"| mysql -udebian-sys-maint -p$MYPW
            echo "FLUSH PRIVILEGES;"| mysql -udebian-sys-maint -p$MYPW
        fi
        if [[ $CHECK_MYSQL_USER_ALL == "NONE" ]]
        then
            echo "CREATE USER '$MYSQL_USER'@'%' IDENTIFIED BY '$MYSQL_PWD';"| mysql -udebian-sys-maint -p$MYPW
            echo "GRANT ALL ON *.* TO '$MYSQL_USER'@'%';"| mysql -udebian-sys-maint -p$MYPW
            echo "FLUSH PRIVILEGES;"| mysql -udebian-sys-maint -p$MYPW
        fi
        sed -i '47d' /etc/mysql/my.cnf
        mysql -h 127.0.0.1 -u $MYSQL_USER -p$MYSQL_PWD < /home/${HCE_USER}/hce-node-bundle/api/python/ini/dc_sites.sql
        mysql -h 127.0.0.1 -u $MYSQL_USER -p$MYSQL_PWD< /home/${HCE_USER}/hce-node-bundle/api/python/ini/dc_urls.sql
        mysql -h 127.0.0.1 -u $MYSQL_USER -p$MYSQL_PWD < /home/${HCE_USER}/hce-node-bundle/api/python/ini/dc_sites_default.sql
        infoDialog "Setup mysql (daemon, databases, user, permissions)" "Databases successfully created" && clear
    else
        infoDialog "Setup mysql (daemon, databases, user, permissions)" "Mysql server is not worked. User and databases not created" && clear
    fi
}

function kvStorage() {
    cp /home/${HCE_USER}/hce-node-bundle/api/python/ini/db-task_keyvalue_template.db /home/${HCE_USER}/hce-node-bundle/api/python/data/dc_dbdata/0.db 2>$TEMPFILE
    infoDialog "Creating kv storage for default site" "Done\n $(<$TEMPFILE)" 8 && clear
}

function checkZmqSo() {
    #if [[ $(grep 'extension=zmq.so' /etc/php5/cli/php.ini) ]]
    if [[ $(php -m | grep zmq) ]]
    then
        echo 'Y'
    else
        exit 1
    fi
}

function checkLocale() {
    if [[ $(locale -a 2>/dev/null | grep 'en_US.utf8') ]]
    then
        echo 'OK'
    fi
    exit 1
}

# END OF FUNCTIONS #


# START #

# CHECK DIALOG INSTALL #
if [[ -z $(which $DIALOG) ]]
then
    if [[ -f '/etc/debian_version' ]]
    then
        echo -e $RED"Dialog not installed, please install dialog using ("$BLUE"sudo apt-get install dialog"$RED") and run "$BLUE"$0"$RED" again"$DEF
        exit 1
    elif [[ -f '/etc/redhat-release' || '/etc/centos-release' ]]
    then
        echo -e $RED"Dialog not installed, please install dialog ("$BLUE"sudo yum install dialog"$RED") and run "$BLUE"$0"$RED" again"$DEF
        exit 1
    else
        echo -e $RED"Dialog not installed, please install dialog and run "$BLUE"$0"$RED" again"$DEF
        exit 1
    fi
fi

if [[ $1 ]]
then
    HCE_USER=$1
else
    HCE_USER=hce
fi

checkSudo
checkOs
checkHomeDir ${HCE_USER}
start "Select components"

for CHOISE in ${CHECK[@]}
do
    case $CHOISE in
        '"0"')
            locales;;
        '"1"')
            dtse_php;;
        '"2"')
            dtse_dc;;
        '"3"')
            dtse_python;;
        '"4"')
            pyzmq;;
        '"5"')
            phpmyadmin;;
        '"6"')
            mvTestSite;;
        '"7"')
            mysqlSetup;;
        '"8"')
            kvStorage;;
    esac
done

infoDialog "Finish" "All selected dependencies installed" && clear

exit 0
